package dk.nsp.mingraviditet.ui.main.knowledge

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.LexiconCategory
import dk.nsp.domain.repository.LexiconRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.adapterviews.HorizontalRecyclerSectionData
import dk.nsp.mingraviditet.ui.adapterviews.HorizontalRecyclerSectionView
import dk.nsp.mingraviditet.ui.common.SectionHeaderAdapterView
import dk.nsp.mingraviditet.ui.common.TextModel
import dk.nsp.mingraviditet.ui.main.knowledge.articles.ArticleCard
import dk.nsp.mingraviditet.ui.main.knowledge.search.SearchNoResultsView
import dk.nsp.mingraviditet.ui.main.knowledge.viewbinding.ArticleCardBindingModel
import dk.nsp.mingraviditet.ui.main.knowledge.viewbinding.CategoryBindingModel
import dk.nsp.mingraviditet.ui.main.knowledge.viewbinding.ChecklistOverviewCardViewBindingModel
import dk.nsp.mingraviditet.util.android.motionlayout.MotionLayoutStateEnum
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.livedata.CombinedLiveData
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import timber.log.Timber

class KnowledgeViewModel @ViewModelInject constructor(
    private val lexiconRepo: LexiconRepository,
    @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    var toolbarState: MotionLayoutStateEnum = MotionLayoutStateEnum.Start

    private val _onChecklistClicked: MutableLiveData<SingleEvent<Checklist>> = MutableLiveData()
    val onChecklistClicked: LiveData<SingleEvent<Checklist>> = _onChecklistClicked

    private val _onArticleCategoryClicked: MutableLiveData<SingleEvent<LexiconCategory>> =
        MutableLiveData()
    val onArticleCategoryClicked: LiveData<SingleEvent<LexiconCategory>> = _onArticleCategoryClicked

    private val _searchResults: MutableLiveData<List<RecyclerAdapterView<*>>> = MutableLiveData()
    val searchResults: LiveData<List<RecyclerAdapterView<*>>> = _searchResults

    private val _categoriesData: MutableLiveData<List<RecyclerAdapterView<*>>> = MutableLiveData()
    private val _checklistsData: MutableLiveData<RecyclerAdapterView<*>> =
        MediatorLiveData<RecyclerAdapterView<*>>().apply {
            addSource(lexiconRepo.checklists) {
                mapChecklistIntoSection(it)
            }
        }

    private val _onError: MutableLiveData<SingleEvent<String>> = MutableLiveData()
    val onError: LiveData<SingleEvent<String>> = _onError

    val onDataCombined =
        CombinedLiveData(lexiconRepo.checklists, _categoriesData) { checklists, categories ->
            val mList = mutableListOf<RecyclerAdapterView<*>>()
            checklists?.let {
                mList.add(mapChecklistIntoSection(it))
            }
            categories?.let {
                mList.add(SectionHeaderAdapterView(TextModel.from(R.string.subjects_title)))
                mList.addAll(it)
            }

            Timber.d("Combined data set -- ${mList.size}")
            mList
        }

    init {
        getChecklists()
        getCategories()
    }

    private fun getChecklists() = viewModelScope.launchIO {
        val checklistsDef = lexiconRepo.getChecklists(this).await()

        when (checklistsDef) {
            is Success -> {
//                _checklistsData.postValue(mapChecklistIntoSection(checklistRes.value))
            }
            is Failed -> {
                _onError.postValue(
                    (checklistsDef.error.message ?: "Kunne ikke hente tjeklister").toSingleEvent()
                )
            }
        }
    }

    private fun getCategories() = viewModelScope.launchIO {
        val categoriesDef = lexiconRepo.getCategories(this)
        when (val categoriesRes = categoriesDef.await()) {
            is Success -> {
                val cards = mapCategoriesIntoSection(categoriesRes.value)

                _categoriesData.postValue(cards)
            }
            is Failed -> {
                _onError.postValue(
                    (categoriesRes.error.message ?: "Kunne ikke hente kategorier").toSingleEvent()
                )
            }
        }
    }

    private fun mapCategoriesIntoSection(categories: List<LexiconCategory>): List<CategoryCard> {
        return categories.map {
            CategoryCard(CategoryBindingModel(it) {
                _onArticleCategoryClicked.postValue(it.toSingleEvent())
            })
        }
    }

    private fun mapChecklistIntoSection(checklists: List<Checklist>): HorizontalRecyclerSectionView {
        val checklistCards = checklists.map { checklist ->
            ChecklistOverviewCard(ChecklistOverviewCardViewBindingModel(checklist) {
                _onChecklistClicked.postValue(checklist.toSingleEvent())
            })
        }

        return HorizontalRecyclerSectionView(
            HorizontalRecyclerSectionData(
                TextModel.from(R.string.checklist_title),
                checklistCards
            )
        )
    }

    fun searchFor(searchTerm: String) {
        if (searchTerm.isBlank()) {
            // Post its own values, so we recreate the list
            val categoryData = _categoriesData.value
            val checklistData = _checklistsData.value

            _categoriesData.postValue(categoryData)
            _checklistsData.postValue(checklistData)
//        } else if (searchTerm.length >= 1) {
        } else if (searchTerm.length >= 3) {
            viewModelScope.launchIO {
                Timber.d("Do search thing")
                val searchRes = lexiconRepo.searchForArticles(this, searchTerm).await()

                when (searchRes) {
                    is Success -> {
                        Timber.d("Found: $searchRes")
                        val cards = searchRes.value.content?.map {
                            ArticleCard(ArticleCardBindingModel(it) {})
                        }

                        if (cards.isNullOrEmpty()) {
                            // Empty state
                            _searchResults.postValue(listOf(SearchNoResultsView()))
                        } else {
                            _searchResults.postValue(cards)
                        }
                    }
                    is Failed -> {

                    }
                }

            }
        }
    }
}