package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dagger.hilt.android.scopes.ActivityScoped
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.domain.repository.LexiconRepository
import dk.nsp.mingraviditet.domain.ExpandCollapseEnum
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success

@ActivityScoped
class ChecklistDetailsViewModel @ViewModelInject constructor(
    private val lexiconRepository: LexiconRepository,
    @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    private val _onError: MutableLiveData<SingleEvent<String>> =
        MutableLiveData<SingleEvent<String>>()
    val onError: LiveData<SingleEvent<String>> = _onError

    val checklist: LiveData<Checklist> = MediatorLiveData<Checklist>().apply {
        addSource(lexiconRepository.checklists) {
            it.firstOrNull { it.id == savedStateHandle.get("checklistId") }
                ?.let { currentChecklist ->
                    postValue(currentChecklist)
                    checklistAdapterViews.postValue(createCards(currentChecklist))
                }
        }
    }

    private val _onExpandCollapse: MutableLiveData<ExpandCollapseEnum> = MutableLiveData()
    val onExpandCollapse: LiveData<ExpandCollapseEnum> = _onExpandCollapse

    fun toggleExpandCollapse() {
        val expanded = _onExpandCollapse.value ?: ExpandCollapseEnum.Collapsed
        _onExpandCollapse.postValue(expanded.toggle())
    }

    val checklistAdapterViews: MutableLiveData<List<RecyclerAdapterView<*>>> = MutableLiveData()

    private val _expandedCards: MutableList<String> = mutableListOf()

    private fun createCards(checklist: Checklist): List<ChecklistCard> {
        return checklist.items.map {
            ChecklistCard(ChecklistCardData(it,
                _expandedCards.contains(it.id),
                onExpandCollapse = { expanded ->
                    if (expanded == ExpandCollapseEnum.Expanded) {
                        _expandedCards.add(it.id)
                    } else {
                        _expandedCards.remove(it.id)
                    }
                }, onCheck = {
                    toggleChecklistItem(checklist, it)
                })
            )
        }
    }

    private fun toggleChecklistItem(checklist: Checklist, checklistItem: ChecklistItem) {
        viewModelScope.launchIO {
            val updatedChecklistItem =
                lexiconRepository.toggleChecklistItem(this, checklist, checklistItem).await()
            when (updatedChecklistItem) {
                is Success -> {
                }
                is Failed -> {
                    _onError.postValue(
                        (updatedChecklistItem.error.message ?: "Fejl").toSingleEvent()
                    )
                }
            }
        }
    }

    fun checklistHasDescription(): Boolean {
        return checklist.value?.description != null
    }
}