package dk.nsp.domain.repository

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import dk.nsp.api.getBackendApiLayer
import dk.nsp.domain.knowledge.*
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.mapSuccess
import dk.nsp.shared.util.asyncDefault
import dk.nsp.shared.util.extensions.replace
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred

class LexiconRepository(
    context: Context
) {
    private val lexiconController = context.getBackendApiLayer().lexiconController

    private val _checklists: MutableLiveData<List<Checklist>> = MutableLiveData()
    val checklists: LiveData<List<Checklist>> = _checklists

    fun getCategories(scope: CoroutineScope): Deferred<ResultWrapper<List<LexiconCategory>>> =
        scope.asyncDefault {
            val categoriesRes = lexiconController.getLexiconCategories(this).await()

            categoriesRes.mapSuccess {
                it.toDomainModel()
            }
        }

    fun getArticlesFor(
        scope: CoroutineScope,
        category: LexiconCategory
    ): Deferred<ResultWrapper<ArticleCatalogue>> = scope.asyncDefault {
        val catalogue = lexiconController.getArticlesFor(this, category.id).await()

        catalogue.mapSuccess {
            it.toDomainModel()
        }
    }

    fun getChecklists(scope: CoroutineScope): Deferred<ResultWrapper<List<Checklist>>> =
        scope.asyncDefault {
            val checklistRes = lexiconController.getChecklists(this).await()

            checklistRes.mapSuccess {
                val domainModel = it.toDomainModel()

                _checklists.postValue(domainModel)

                domainModel
            }
        }

    fun toggleChecklistItem(
        scope: CoroutineScope,
        checklist: Checklist,
        checklistItem: ChecklistItem
    ): Deferred<ResultWrapper<ChecklistItem>> = scope.asyncDefault {
        val toggleRes =
            lexiconController.toggleChecklistItem(
                this,
                checklist.toServerModel(),
                checklistItem.toServerModel()
            ).await()

        toggleRes.mapSuccess {
            // Instead of getting all the checklists again, we just update our local LiveData
            val copyOfChecklistItems = checklist.items.toMutableList()
            val updatedListOfChecklistItems =
                copyOfChecklistItems.replace(checklistItem, it.toDomainModel())
            val copyOfChecklist = checklist.copy(items = updatedListOfChecklistItems)
            // Replace current list with new list
            val updatedListOfChecklists =
                _checklists.value?.toMutableList()?.replace(checklist, copyOfChecklist)
            //Post updated list to our local LiveData
            _checklists.postValue(updatedListOfChecklists)

            it.toDomainModel()
        }
    }

    fun searchForArticles(
        scope: CoroutineScope,
        searchQuery: String
    ): Deferred<ResultWrapper<ArticleCatalogue>> = scope.asyncDefault {
        val searchRes = lexiconController.searchForArticles(this, searchQuery).await()

        searchRes.mapSuccess {
            it.toDomainModel()
        }
    }
}